"use strict";
var RamlWrapper = require("./raml10parser");
/**
 * @hidden
 * Build Wrapper node corresponding to the High Level node
 **/
function buildWrapperNode(node, setAsTopLevel) {
    if (setAsTopLevel === void 0) { setAsTopLevel = true; }
    var definition = node.definition();
    var nodeClassName = definition.nameId();
    var wrapperConstructor = classMap[nodeClassName];
    if (!wrapperConstructor) {
        var priorities = determineSuperclassesPriorities(definition);
        var superTypes = definition.allSuperTypes().sort(function (x, y) { return priorities[x.nameId()] - priorities[y.nameId()]; });
        var wr = null;
        for (var i = 0; i < superTypes.length; i++) {
            var superTypeName = superTypes[i].nameId();
            wrapperConstructor = classMap[superTypeName];
            if (superTypeName == "DataElement") {
                wr = superTypeName;
                //This is only case of nested hierarchy
                continue;
            }
            if (superTypeName == "RAMLLanguageElement") {
                //depth first
                continue;
            }
            if (wrapperConstructor) {
                break;
            }
        }
        if (!wrapperConstructor) {
            wr = superTypeName;
        }
    }
    if (!wrapperConstructor) {
        wrapperConstructor = classMap["RAMLLanguageElement"];
    }
    return wrapperConstructor(node, setAsTopLevel);
}
exports.buildWrapperNode = buildWrapperNode;
function determineSuperclassesPriorities(td, priorities, path) {
    if (priorities === void 0) { priorities = {}; }
    if (path === void 0) { path = {}; }
    var typeName = td.nameId();
    if (path[typeName]) {
        return;
    }
    path[typeName] = true;
    var rank = (priorities[typeName] != null && priorities[typeName] + 1) || 0;
    var superTypes = td.superTypes();
    superTypes.forEach(function (x) {
        var name = x.nameId();
        var r = priorities[name];
        if (r == null || rank > r) {
            priorities[name] = rank;
            determineSuperclassesPriorities(x, priorities, path);
        }
    });
    delete path[typeName];
    return priorities;
}
var classMap = {
    "AbstractSecurityScheme": function (x, y) { return new RamlWrapper.AbstractSecuritySchemeImpl(x, y); },
    "AnnotationRef": function (x) { return new RamlWrapper.AnnotationRefImpl(x); },
    "AnnotationTarget": function (x) { return new RamlWrapper.AnnotationTargetImpl(x); },
    "AnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.AnnotationTypeDeclarationImpl(x, y); },
    "AnyType": function (x) { return new RamlWrapper.AnyTypeImpl(x); },
    "Api": function (x, y) { return new RamlWrapper.ApiImpl(x, y); },
    "ArrayAnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.ArrayAnnotationTypeDeclarationImpl(x, y); },
    "ArrayTypeDeclaration": function (x, y) { return new RamlWrapper.ArrayTypeDeclarationImpl(x, y); },
    "BasicSecurityScheme": function (x, y) { return new RamlWrapper.BasicSecuritySchemeImpl(x, y); },
    "BooleanAnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.BooleanAnnotationTypeDeclarationImpl(x, y); },
    "BooleanType": function (x) { return new RamlWrapper.BooleanTypeImpl(x); },
    "BooleanTypeDeclaration": function (x, y) { return new RamlWrapper.BooleanTypeDeclarationImpl(x, y); },
    "ContentType": function (x) { return new RamlWrapper.ContentTypeImpl(x); },
    "CustomSecurityScheme": function (x, y) { return new RamlWrapper.CustomSecuritySchemeImpl(x, y); },
    "DateTypeAnnotationDeclaration": function (x, y) { return new RamlWrapper.DateTypeAnnotationDeclarationImpl(x, y); },
    "DateTypeDeclaration": function (x, y) { return new RamlWrapper.DateTypeDeclarationImpl(x, y); },
    "DigestSecurityScheme": function (x, y) { return new RamlWrapper.DigestSecuritySchemeImpl(x, y); },
    "DocumentationItem": function (x, y) { return new RamlWrapper.DocumentationItemImpl(x, y); },
    "ExampleSpec": function (x, y) { return new RamlWrapper.ExampleSpecImpl(x, y); },
    "ExampleString": function (x) { return new RamlWrapper.ExampleStringImpl(x); },
    "Extension": function (x, y) { return new RamlWrapper.ExtensionImpl(x, y); },
    "FileTypeDeclaration": function (x, y) { return new RamlWrapper.FileTypeDeclarationImpl(x, y); },
    "FixedUriString": function (x) { return new RamlWrapper.FixedUriStringImpl(x); },
    "FullUriTemplateString": function (x) { return new RamlWrapper.FullUriTemplateStringImpl(x); },
    "GlobalSchema": function (x, y) { return new RamlWrapper.GlobalSchemaImpl(x, y); },
    "HasNormalParameters": function (x, y) { return new RamlWrapper.HasNormalParametersImpl(x, y); },
    "ImportDeclaration": function (x, y) { return new RamlWrapper.ImportDeclarationImpl(x, y); },
    "IntegerTypeDeclaration": function (x, y) { return new RamlWrapper.IntegerTypeDeclarationImpl(x, y); },
    "JSonSchemaString": function (x) { return new RamlWrapper.JSonSchemaStringImpl(x); },
    "Library": function (x, y) { return new RamlWrapper.LibraryImpl(x, y); },
    "LibraryBase": function (x, y) { return new RamlWrapper.LibraryBaseImpl(x, y); },
    "LocationKind": function (x) { return new RamlWrapper.LocationKindImpl(x); },
    "MarkdownString": function (x) { return new RamlWrapper.MarkdownStringImpl(x); },
    "Method": function (x, y) { return new RamlWrapper.MethodImpl(x, y); },
    "MethodBase": function (x, y) { return new RamlWrapper.MethodBaseImpl(x, y); },
    "MimeType": function (x) { return new RamlWrapper.MimeTypeImpl(x); },
    "ModelLocation": function (x) { return new RamlWrapper.ModelLocationImpl(x); },
    "NumberAnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.NumberAnnotationTypeDeclarationImpl(x, y); },
    "NumberType": function (x) { return new RamlWrapper.NumberTypeImpl(x); },
    "NumberTypeDeclaration": function (x, y) { return new RamlWrapper.NumberTypeDeclarationImpl(x, y); },
    "OAuth1SecurityScheme": function (x, y) { return new RamlWrapper.OAuth1SecuritySchemeImpl(x, y); },
    "OAuth1SecuritySchemeSettings": function (x, y) { return new RamlWrapper.OAuth1SecuritySchemeSettingsImpl(x, y); },
    "OAuth2SecurityScheme": function (x, y) { return new RamlWrapper.OAuth2SecuritySchemeImpl(x, y); },
    "OAuth2SecuritySchemeSettings": function (x, y) { return new RamlWrapper.OAuth2SecuritySchemeSettingsImpl(x, y); },
    "ObjectAnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.ObjectAnnotationTypeDeclarationImpl(x, y); },
    "ObjectTypeDeclaration": function (x, y) { return new RamlWrapper.ObjectTypeDeclarationImpl(x, y); },
    "Overlay": function (x, y) { return new RamlWrapper.OverlayImpl(x, y); },
    "PassThroughSecurityScheme": function (x, y) { return new RamlWrapper.PassThroughSecuritySchemeImpl(x, y); },
    "PassThroughSecuritySchemeSettings": function (x, y) { return new RamlWrapper.PassThroughSecuritySchemeSettingsImpl(x, y); },
    "RAMLLanguageElement": function (x, y) { return new RamlWrapper.RAMLLanguageElementImpl(x, y); },
    "RAMLSimpleElement": function (x, y) { return new RamlWrapper.RAMLSimpleElementImpl(x, y); },
    "Reference": function (x) { return new RamlWrapper.ReferenceImpl(x); },
    "RelativeUriString": function (x) { return new RamlWrapper.RelativeUriStringImpl(x); },
    "Resource": function (x, y) { return new RamlWrapper.ResourceImpl(x, y); },
    "ResourceBase": function (x, y) { return new RamlWrapper.ResourceBaseImpl(x, y); },
    "ResourceType": function (x, y) { return new RamlWrapper.ResourceTypeImpl(x, y); },
    "ResourceTypeRef": function (x) { return new RamlWrapper.ResourceTypeRefImpl(x); },
    "Response": function (x, y) { return new RamlWrapper.ResponseImpl(x, y); },
    "SchemaString": function (x) { return new RamlWrapper.SchemaStringImpl(x); },
    "SecuritySchemePart": function (x, y) { return new RamlWrapper.SecuritySchemePartImpl(x, y); },
    "SecuritySchemeRef": function (x) { return new RamlWrapper.SecuritySchemeRefImpl(x); },
    "SecuritySchemeSettings": function (x, y) { return new RamlWrapper.SecuritySchemeSettingsImpl(x, y); },
    "StatusCodeString": function (x) { return new RamlWrapper.StatusCodeStringImpl(x); },
    "StringAnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.StringAnnotationTypeDeclarationImpl(x, y); },
    "StringType": function (x) { return new RamlWrapper.StringTypeImpl(x); },
    "StringTypeDeclaration": function (x, y) { return new RamlWrapper.StringTypeDeclarationImpl(x, y); },
    "Trait": function (x, y) { return new RamlWrapper.TraitImpl(x, y); },
    "TraitRef": function (x) { return new RamlWrapper.TraitRefImpl(x); },
    "TypeDeclaration": function (x, y) { return new RamlWrapper.TypeDeclarationImpl(x, y); },
    "UnionAnnotationTypeDeclaration": function (x, y) { return new RamlWrapper.UnionAnnotationTypeDeclarationImpl(x, y); },
    "UnionTypeDeclaration": function (x, y) { return new RamlWrapper.UnionTypeDeclarationImpl(x, y); },
    "UriTemplate": function (x) { return new RamlWrapper.UriTemplateImpl(x); },
    "ValueType": function (x) { return new RamlWrapper.ValueTypeImpl(x); },
    "XMLSchemaString": function (x) { return new RamlWrapper.XMLSchemaStringImpl(x); }
};
//# sourceMappingURL=raml10factory.js.map